// Bar Range Indicator
// Last changed on 2009-05-11
// Part of Forex Strategy Builder v2.8.3.8+
// Website http://forexsb.com/
// Copyright (c) 2006 - 2009 Miroslav Popov - All rights reserved.
// This code or any part of it cannot be used in other applications without a permission.

using System;
using System.Drawing;

namespace Forex_Strategy_Builder
{
    /// <summary>
    /// Bar Range Bands Indicator
	/// Draws the high and low bands on the Price Chart
	/// filters entering or exiting positions based on close within pips or percentage of high or low band
	/// allow open and closing position with high or low -- based on value from previous bar's uper and lower band
	/// to simulate what would happen if the high or low actually pushed the band up or down
    /// </summary>
    public class BarRangeBands : Indicator
    {
        /// <summary>
        /// Sets the default indicator parameters for the designated slot type
        /// </summary>
        public BarRangeBands(SlotTypes slotType)
        {
            // General properties
            IndicatorName  = "Bar Range Bands";
            PossibleSlots  = SlotTypes.Open | SlotTypes.Close | SlotTypes.OpenFilter | SlotTypes.CloseFilter;
			CustomIndicator = true;

            // Setting up the indicator parameters
            IndParam = new IndicatorParam();
            IndParam.IndicatorName = IndicatorName;
            IndParam.SlotType      = slotType;

            // The ComboBox parameters
            IndParam.ListParam[0].Caption  = "Direction";
            IndParam.ListParam[0].ItemList = new string[]
			{
				"Continuation Preference",
				"Reversal Preference"
			};
            IndParam.ListParam[0].Index    = 0;
            IndParam.ListParam[0].Text     = IndParam.ListParam[0].ItemList[IndParam.ListParam[0].Index];
            IndParam.ListParam[0].Enabled  = true;
			if (slotType == SlotTypes.CloseFilter || slotType == SlotTypes.Close)
			{
	           IndParam.ListParam[0].ToolTip  = "Continuation: exit long when near low band, exit short when near high band \nReversal: exit long when near high band, exit short when near low band";
			}
			else
			{
	           IndParam.ListParam[0].ToolTip  = "Continuation: go long when near high band, go short when near low band \nReversal: go long when near low band, go short when near high band";
			}
			

			IndParam.ListParam[1].Caption  = "Basis";
			if (slotType == SlotTypes.Open || slotType == SlotTypes.Close) {
				IndParam.ListParam[1].ItemList = new string[]
				{
					"Current Bar"
				};
				IndParam.ListParam[1].ToolTip  = "Basis of price component near upper or lower band.\nUses upper and lower band values from previous bar.";
			}
			else {
				IndParam.ListParam[1].ItemList = new string[]
				{
					"Close",
					"High/Low",
					"Open"
				};
				IndParam.ListParam[1].ToolTip  = "Basis of price component near upper or lower band.";
			}
			IndParam.ListParam[1].Index    = 0;
			IndParam.ListParam[1].Text     = IndParam.ListParam[1].ItemList[IndParam.ListParam[1].Index];
			IndParam.ListParam[1].Enabled  = true;

            // The NumericUpDown parameters
            IndParam.NumParam[0].Caption = "Number of bars";
            IndParam.NumParam[0].Value   = 12;
            IndParam.NumParam[0].Min     = 1;
            IndParam.NumParam[0].Max     = 200;
            IndParam.NumParam[0].Enabled = true;
            IndParam.NumParam[0].ToolTip = "The number of bars to calculate the range.";


            IndParam.NumParam[1].Caption = "Vertical Shift";
            IndParam.NumParam[1].Value   = 0;
            IndParam.NumParam[1].Min     = 0;
            IndParam.NumParam[1].Max     = 5000;
            IndParam.NumParam[1].Point   = 0;
            IndParam.NumParam[1].Enabled = true;
            IndParam.NumParam[1].ToolTip = "Price must be within this range from high/low band to enter.";

            IndParam.NumParam[2].Caption = "Min Bar Range";
            IndParam.NumParam[2].Value   = 0;
            IndParam.NumParam[2].Min     = 0;
            IndParam.NumParam[2].Max     = 5000;
            IndParam.NumParam[2].Point   = 0;
            IndParam.NumParam[2].Enabled = true;
            IndParam.NumParam[2].ToolTip = "Minimum Bar Range to use Vertical Shift parameter.";


            // The CheckBox parameters
            IndParam.CheckParam[0].Caption = "Use previous bar value";
            IndParam.CheckParam[0].Checked = PrepareUsePrevBarValueCheckBox(slotType);
            IndParam.CheckParam[0].Enabled = true;
            IndParam.CheckParam[0].ToolTip = "Use the indicator value from the previous bar.";

            IndParam.CheckParam[1].Caption = "Shift as Percent";
            IndParam.CheckParam[1].Checked = false;
            IndParam.CheckParam[1].Enabled = true;
            IndParam.CheckParam[1].ToolTip = "Check to set Vertical Shift as percentage of Bar Range; unchecked uses value in pips.";


            return;
        }

        /// <summary>
        /// Calculates the indicator's components
        /// </summary>
        public override void Calculate(SlotTypes slotType)
        {
            // Reading the parameters
            int    nBars  = (int)IndParam.NumParam[0].Value;
            double dEnteredShift = IndParam.NumParam[1].Value;
			double dMinBarRange = IndParam.NumParam[2].Value;
            int    iPrvs  = IndParam.CheckParam[0].Checked ? 1 : 0;
			bool usePercent = IndParam.CheckParam[1].Checked ? true : false;

            // Calculation
            int iFirstBar = nBars + 1;

			double[] adHighs = new double[Bars];
			double[] adLows = new double[Bars];
			double[] adBarRange = new double[Bars];
			double[] adVertShift = new double[Bars];
			double[] adNearHigh = new double[Bars];
			double[] adNearLow = new double[Bars];

            for (int iBar = iFirstBar; iBar < Bars; iBar++)
            {
				// get High and Low band
                double maxHigh = double.MinValue;
                double minLow  = double.MaxValue;
				double dVertShift = 0;
				double dBarRange;

                for (int i = 0; i < nBars; i++)
                {
                    if (High[iBar - i] > maxHigh) {
                        maxHigh = High[iBar - i];
					}
                    if (Low[iBar - i] < minLow) {
                        minLow = Low[iBar - i];
					}
                }
                adHighs[iBar] = maxHigh;
				adLows[iBar] = minLow;

				// get bar range, if over minimum, then figure vertical shift as percentage of bar range, or as pips
				// don't use bankers rounding
				dBarRange = Math.Round((maxHigh - minLow) / Point, MidpointRounding.AwayFromZero);
				if (usePercent)
				{
					dVertShift = Math.Round((dBarRange * dEnteredShift)/100, MidpointRounding.AwayFromZero);
				}
				else
				{
					dVertShift = dEnteredShift;
				}
				dVertShift *= Point;


				adBarRange[iBar] = dBarRange;
				// convert from pip value back to integer value for display in component
				adVertShift[iBar] = dVertShift/Point;


				if (dBarRange >= dMinBarRange)
				{					
					// figure if price basis is near High or Low band
					// if slot type is OpenFilter or CloseFilter, set to 1 so AllowOpenPosition will work
					// if slot type is Open or Close, use band value from previous bar so OpenLongPrice or CloseLongPrice will work
					if (slotType == SlotTypes.OpenFilter || slotType == SlotTypes.CloseFilter)
					{
						switch (IndParam.ListParam[1].Text)
						{
							case "Open":
								if (Math.Abs(maxHigh - Open[iBar]) <= dVertShift)
								{
									adNearHigh[iBar] = 1.0;
								}
								if (Math.Abs(minLow - Open[iBar]) <= dVertShift)
								{
									adNearLow[iBar] = 1.0;
								}
								break;
							case "Close":
								if (Math.Abs(maxHigh - Close[iBar]) <= dVertShift)
								{
									adNearHigh[iBar] = 1.0;
								}
								if (Math.Abs(minLow - Close[iBar]) <= dVertShift)
								{
									adNearLow[iBar] = 1.0;
								}
								break;
							case "High/Low": 
								if (Math.Abs(maxHigh - High[iBar]) <= dVertShift)
								{
									adNearHigh[iBar] = 1.0;
								}
								if (Math.Abs(minLow - Low[iBar]) <= dVertShift)
								{
									adNearLow[iBar] = 1.0;
								}
								break;
						}
					}
					else if (slotType == SlotTypes.Open || slotType == SlotTypes.Close)
					{
							// for High/Low, use band value from previous bar
							// adjusted for vertical shift
							adNearHigh[iBar] = adHighs[iBar-1] - dVertShift;
							adNearLow[iBar] = adLows[iBar-1] + dVertShift;
					}

				}



            }
            
            // Saving the components
            Component = new IndicatorComp[6];

            Component[0] = new IndicatorComp();
            Component[0].CompName  = "High Band";
            Component[0].DataType  = IndComponentType.IndicatorValue;
            Component[0].ChartType = IndChartType.Line;
			Component[0].ChartColor	= Color.Green;
            Component[0].FirstBar  = iFirstBar;
            Component[0].Value     = adHighs;

            Component[1] = new IndicatorComp();
            Component[1].CompName  = "Low Band";
            Component[1].DataType  = IndComponentType.IndicatorValue;
            Component[1].ChartType = IndChartType.Line;
			Component[1].ChartColor	= Color.Red;
            Component[1].FirstBar  = iFirstBar;
            Component[1].Value     = adLows;

            Component[2] = new IndicatorComp();
			Component[2].CompName = "Bar Range";
			Component[2].DataType  = IndComponentType.IndicatorValue;
            Component[2].ChartType = IndChartType.NoChart;
            Component[2].FirstBar  = iFirstBar;
            Component[2].Value     = adBarRange;

            Component[3] = new IndicatorComp();
			Component[3].CompName = "Vertical Shift in pips";
			Component[3].DataType  = IndComponentType.IndicatorValue;
            Component[3].ChartType = IndChartType.NoChart;
            Component[3].FirstBar  = iFirstBar;
            Component[3].Value     =adVertShift;


            Component[4] = new IndicatorComp();
            Component[4].ChartType = IndChartType.NoChart;
            Component[4].FirstBar  = iFirstBar;
            

            Component[5] = new IndicatorComp();
            Component[5].ChartType = IndChartType.NoChart;
            Component[5].FirstBar  = iFirstBar;
            


            // Sets the Component's type
            if (slotType == SlotTypes.OpenFilter)
            {
				if (IndParam.ListParam[0].Text == "Continuation Preference")
				{
					Component[4].Value     = adNearHigh;
					Component[4].DataType = IndComponentType.AllowOpenLong;
					Component[4].CompName = "Is long entry allowed";
					Component[5].Value     =adNearLow;
					Component[5].DataType = IndComponentType.AllowOpenShort;
					Component[5].CompName = "Is short entry allowed";
				}
				else if (IndParam.ListParam[0].Text == "Reversal Preference")
				{
					Component[4].Value     = adNearLow;
					Component[4].DataType = IndComponentType.AllowOpenLong;
					Component[4].CompName = "Is long entry allowed";
					Component[5].Value     =adNearHigh;
					Component[5].DataType = IndComponentType.AllowOpenShort;
					Component[5].CompName = "Is short entry allowed";
				}

            }
            else if (slotType == SlotTypes.CloseFilter)
            {
				if (IndParam.ListParam[0].Text == "Continuation Preference")
				{
					Component[4].Value     = adNearLow;
					Component[4].DataType = IndComponentType.ForceCloseLong;
					Component[4].CompName = "Close out long position";
					Component[5].Value     =adNearHigh;
					Component[5].DataType = IndComponentType.ForceCloseShort;
					Component[5].CompName = "Close out short position";
				}
				else if (IndParam.ListParam[0].Text == "Reversal Preference")
				{
					Component[4].Value     = adNearHigh;
					Component[4].DataType = IndComponentType.ForceCloseLong;
					Component[4].CompName = "Close out long position";
					Component[5].Value     =adNearLow;
					Component[5].DataType = IndComponentType.ForceCloseShort;
					Component[5].CompName = "Close out short position";
				}
			}
			else if (slotType == SlotTypes.Open)
            {
				if (IndParam.ListParam[0].Text == "Continuation Preference")
				{
					Component[4].Value     = adNearHigh;
					Component[4].DataType = IndComponentType.OpenLongPrice;
					Component[4].CompName = "Long position entry price";
					Component[5].Value     =adNearLow;
					Component[5].DataType = IndComponentType.OpenShortPrice;
					Component[5].CompName = "Short position entry price";
				}
				else if (IndParam.ListParam[0].Text == "Reversal Preference")
				{
					Component[4].Value     = adNearLow;
					Component[4].DataType = IndComponentType.OpenLongPrice;
					Component[4].CompName = "Long position entry price";
					Component[5].Value     =adNearHigh;
					Component[5].DataType = IndComponentType.OpenShortPrice;
					Component[5].CompName = "Short position entry price";
				}

            }
            else if (slotType == SlotTypes.Close)
            {
				if (IndParam.ListParam[0].Text == "Continuation Preference")
				{
					Component[4].Value     = adNearLow;
					Component[4].DataType = IndComponentType.CloseLongPrice;
					Component[4].CompName = "Long position closing price";
					Component[5].Value     =adNearHigh;
					Component[5].DataType = IndComponentType.CloseShortPrice;
					Component[5].CompName = "Short position closing price";
				}
				else if (IndParam.ListParam[0].Text == "Reversal Preference")
				{
					Component[4].Value     = adNearHigh;
					Component[4].DataType = IndComponentType.CloseLongPrice;
					Component[4].CompName = "Long position closing price";
					Component[5].Value     =adNearLow;
					Component[5].DataType = IndComponentType.CloseShortPrice;
					Component[5].CompName = "Short position closing price";
				}
			}

            return;
        }

        /// <summary>
        /// Sets the indicator logic description
        /// </summary>
        public override void SetDescription(SlotTypes slotType)
        {
            int    nBars       = (int) IndParam.NumParam[0].Value;
            string sVertShift  = IndParam.NumParam[1].ValueToString;
            string sBarMin = IndParam.NumParam[2].ValueToString;
			string basis = IndParam.ListParam[1].Text;
			string percPips = IndParam.CheckParam[1].Checked ? "%" : " pips";

			EntryFilterLongDescription  = "the " + basis + " is within " + sVertShift  + percPips + " of the ";
			EntryFilterShortDescription = "the " + basis + " is within " + sVertShift  + percPips + " of the ";
			ExitFilterLongDescription   = "the " + basis + " is within " + sVertShift + percPips + " of the ";
			ExitFilterShortDescription  = "the " + basis + " is within " + sVertShift + percPips + " of the ";

            switch (IndParam.ListParam[0].Text)
            {
                case "Continuation Preference":
                    EntryFilterLongDescription  += "Upper Band of the Bar Range of the last " + nBars.ToString() + " bars ";
                    EntryFilterShortDescription += "Lower Band of the Bar Range of the last " + nBars.ToString() + " bars ";
                    ExitFilterLongDescription   += "Lower Band of the Bar Range of the last " + nBars.ToString() + " bars ";
                    ExitFilterShortDescription  += "Upper Band of the Bar Range of the last " + nBars.ToString() + " bars ";
                    break;

                case "Reversal Preference":
                    EntryFilterLongDescription  += "Lower Band of the Bar Range of the last " + nBars.ToString() + " bars ";
                    EntryFilterShortDescription += "Upper Band of the Bar Range of the last " + nBars.ToString() + " bars ";
                    ExitFilterLongDescription   += "Upper Band of the Bar Range of the last " + nBars.ToString() + " bars ";
                    ExitFilterShortDescription  += "Lower Band of the Bar Range of the last " + nBars.ToString() + " bars ";
                    break;

                default:
                    break;
            }

			EntryFilterLongDescription  += ", or within 0 pips if the range is less than " + sBarMin + " pips.";
			EntryFilterShortDescription += ", or within 0 pips if the range is less than " + sBarMin + " pips.";
			ExitFilterLongDescription   += ", or within 0 pips if the range is less than " + sBarMin + " pips.";
			ExitFilterShortDescription  += ", or within 0 pips if the range is less than " + sBarMin + " pips.";

            return;
        }

        /// <summary>
        /// Indicator to string
        /// </summary>
        public override string ToString()
        {
			string percPips = IndParam.CheckParam[1].Checked ? "%" : "pips";
            string sString = IndicatorName +
                (IndParam.CheckParam[0].Checked ? "* (" : " (") + 
				IndParam.SlotType +
                ",n=" + IndParam.NumParam[0].ValueToString + 
				",vertShift=" + 	IndParam.NumParam[1].ValueToString + percPips +
				"," + IndParam.ListParam[0].Text +
				",basis=" + IndParam.ListParam[1].Text +
				")";

            return sString;
        }
    }
}